import { DeleteOptions, GetOptions, ListOptions, PagedItems, ResourceIdentityWithError, SubscriptionResponse, UserInfoResponse, WSSubscriptionResponse } from './common';
import { Configuration } from './config';
import { Policy } from './policy';
import { ServiceResponse } from './response';
import { Service } from './service';
export declare const DefaultOrganizerServiceUri = "https://org-api.connect.trimble.com/v1/";
/**
 * The request body for the @see Organiser#createTree operation.
 */
export interface CreateTreeRequest {
    /** The optional identifier for the new tree, if not provided service will generate an unique identifier. */
    id?: string;
    /** The tree short name. */
    name?: string;
    /** The optional description. */
    description?: string;
    /** The type of the tree that defines the symantic. */
    type: string;
}
/**
 * The request parameters for the @see Organiser#getTree operation.
 */
export interface GetTreeOptions extends GetOptions {
}
/**
 * The request parameters for the @see Organiser#getNode operation.
 */
export interface GetNodeOptions extends GetOptions {
}
/**
 * The request body for the @see Organiser#updateTree operation.
 * All properties are optional, if property is not provided in the request the property will retain it's old value.
 */
export interface UpdateTreeRequest {
    /**
     * The last seen version of the tree descriptor.
     * If provided service will use this information for the concurency control and will refuse to apply changes if current version is not equal to version provided in the request.
     */
    v?: number;
    /** The new name. */
    name?: string;
    /** The new description. */
    description?: string;
    /** The new type. */
    type?: string;
}
/**
 * The tree descriptor returned from @see Organiser#createTree , @see Organiser#updateTree , @see Organiser#deleteTree and @see Organiser#getTree operations.
 */
export interface TreeResponse {
    /** The unique tree identifier. */
    id: string;
    /** The creation timestamp in ISO format. */
    createdAt: string;
    /** The identifier of the principal who created the tree. */
    createdBy: string;
    /** The last modification timestamp in ISO format. */
    modifiedAt: string;
    /** The identifier of the princilat who last modified the tree descriptor. */
    modifiedBy: string;
    /** The current version of the tree descriptor. */
    v: number;
    /** The access conrol policy document attached to the tree. This property is returned only if caller has org:GetTreePolicy right. */
    policy?: Policy;
    /** The deletion marker. If not present or false the tree is active, otherwise the tree is marked as deleted. */
    deleted?: boolean;
    /** The tree short name. */
    name: string;
    /** The optional description. */
    description?: string;
    /** The tree type that defines the symantic. */
    type: string;
}
/**
 * The request body for the @see Organiser#createNode operation.
 */
export interface CreateNodeRequest {
    /** The optional identifier for the new tree, if not provided service will generate an unique identifier. */
    id?: string;
    /** The node name. To pesent a tree-like structure this name is interpreted as a full path in a tree with '/' character as a separator between path segments. */
    name?: string;
    /** The optional collection of links to resources to associate with this node. */
    links?: string[];
}
/**
 * The request body for the @see Organiser#updateNode operation.
 * All properties are optional, if property not provided in the request the property will retain it's old value.
 */
export interface UpdateNodeRequest {
    /**
     * The last seen version of the tree descriptor.
     * If provided service will use this information for the concurency control and will refuse to apply changes if current version is not equal to version provided in the request.
     */
    v?: number;
    /** The new name. */
    name?: string;
    /** The new set of links to resources. If this property is provided it will override all existing links associated. For differential changes use @see add and @see remove properties. */
    links?: string[];
    /** The set of links to add to the associations collection (@see NodeResponse#links ) all currently associated links remain unchanged. */
    add?: string[];
    /** The set of links to be removef from to the associations collection (@see NodeResponse#links ). Links that are not associated already, are ignored. */
    remove?: string[];
}
/**
 * The tree descriptor returned from @see Organiser#createNode, @see Organiser#updateNode, @see Organiser#deleteNode and @see Organiser#getNode operations.
 */
export interface NodeResponse {
    /** The forest identifier. */
    forestId: string;
    /** The tree identifier. */
    treeId: string;
    /** The unique node identifier. */
    id: string;
    /** The creation timestamp in ISO format. */
    createdAt: string;
    /** The identifier of the principal who created the node. */
    createdBy: string;
    /** The last modification timestamp in ISO format. */
    modifiedAt: string;
    /** The identifier of the princilat who last modified the node. */
    modifiedBy: string;
    /** The version of the node. */
    v: number;
    /** The deletion marker. If not present or false the node is active, otherwise the node is marked as deleted. */
    deleted?: boolean;
    /** The name. */
    name: string;
    /**
     * The collection of links to resources to associate with this node. Might be absent if client requested to skip this property from the response.
     * If property is requested and there are no associations then empty array is returned.
     * In case of externalized links this property is not returned.
     */
    links?: string[];
    /** The number of links associated with the node. */
    count: number;
    /** URL to externalized links, if that's the case. This is returned even when the nolinks option was used in the request. */
    linksUrl?: string;
}
/**
 * The de-serialized data returned from the request with list of tree descriptors.
 */
export interface ListTreesResponse extends PagedItems<TreeResponse> {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListNodesResponse extends PagedItems<NodeResponse> {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListTreesOptions extends ListOptions {
}
/**
 * The de-serialized data returned from the request with list of nodes.
 */
export interface ListNodesOptions extends ListOptions {
    /** The entity to serach for in the links collection. If specified only nodes that are linking to that entity are returned. */
    link?: string;
    /** The node name prefix to filter nodes. Only nodes whos name starts with this prefix are returned. Allows conviniently ferch branches of the tree. */
    prefix?: string;
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The de-serialized data returned from the request with list of node versions.
 */
export interface ListNodeVersionsOptions extends ListOptions {
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The tree identity.
 */
export interface TreeIdentity {
    /** The forest identifier. */
    forestId: string;
    /** The tree identifier. */
    treeId: string;
}
/**
 * The node identity.
 */
export interface NodeIdentity {
    /** The forest identifier. */
    forestId?: string;
    /** The tree identifier. */
    treeId?: string;
    /** The node identifier. When only the id is specified then the latest version is requested. */
    nodeId: string;
    /** Requesting the node with this version. Optional. */
    v?: number;
}
/**
 * The batch get API request body.
 */
export interface BatchGetRequest extends OrgBatchGetMap<TreeIdentity, NodeIdentity> {
}
/**
 * The batch get API request body.
 */
export interface BatchGetNodesRequest extends OrgBatchGetMap<TreeIdentity, NodeIdentity> {
}
/**
 * The BatchGet API options.
 */
export interface BatchGetOptions extends GetOptions {
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The response from batch get API.
 */
export interface BatchGetResponse {
    /** The list of unprocessed requested resources.
     * It is important that client has a retry logic to fetch the items unprocessed in the original request.
     * Collection of unprocessed items can be passed as is to the next request.
     */
    unprocessed: OrgBatchGetMap<TreeIdentity, NodeIdentity>;
    /** The collection requested nodes. */
    responses: OrgBatchGetMap<TreeResponse, NodeResponse>;
    /** The list of errors. */
    errors: OrgBatchGetMap<ResourceIdentityWithError<TreeIdentity>, ResourceIdentityWithError<NodeIdentity>>;
}
/**
 * The batch get API request body.
 */
export interface OrgBatchGetMap<T, N> {
    /** The collection of trees. */
    trees?: T[];
    /** The collection of nodes. */
    nodes?: N[];
}
/**
 * The changeset request parameters .
 */
export interface ChangesetOptions {
    /** Controls whether to send list of link toghether with the node descriptors. By default links are sent. App can reduce the payload size when requesting tree structure by using this parameter. */
    nolinks?: boolean;
}
/**
 * The change node request is an upsert type of request.
 * It could be that it creates a new node or updates existing or even deletes.
 */
export declare type ChangeNodeRequest = CreateNodeRequest | UpdateNodeRequest | {
    id: string;
    deleted?: boolean;
};
/**
 * Configures the mode of processing he changeset.
 * The changeset can be applied in snapshot or incremental mode.
 * In the snapshot mode all nodes not mentioned int he requests will be marked as deleted.
 * In the incremental mode nodes not mentioned in the request are not touched.
 */
export interface OrgChangesetConfiguration {
    /** The changeset mode. If not specified the incremental mode is used. */
    snapshot?: boolean;
    /** The node name prefix that will be used as a scope for the changeset. Used only in the snapshot mode to limit the nodes that will be implicitly deleted. */
    scopePrefix?: string;
}
/**
 * The synchronous changeset request.
 * Actual node modification requests in addition to the configration.
 */
export interface OrgChangesetRequest extends OrgChangesetConfiguration {
    /** The collection of nodes to add, update or delete. */
    nodes: ChangeNodeRequest[];
}
/**
 * The response body for @see PSet#changeset operation if it goes to async execution mode (>500 items in the request).
 */
export interface OrgChangesetResponseAccepted {
    /** The changeset identifier. */
    id: string;
    /** The user identifier. */
    createdBy: string;
    /** The creation timestamp in ISO format. */
    createdAt: string;
    /** The changeset processing status: WaitingUpload, Queued, Processing, Done, Failed. */
    status: string;
    /** The changeset mode was used. */
    snapshot: boolean;
    /** The node name prefix was used. */
    scopePrefix?: string;
}
/**
 * The response body for @see Organizer#changeset operation if it is executed in sync mode (<=500 items in the request).
 */
export interface OrgChangesetResponseInline extends OrgChangesetResponseAccepted {
    /** The timestamp in ISO format when processing has been started. */
    startedAt: string;
    /** The timestamp in ISO format when processing has been finished. */
    processedAt: string;
    /** The error message if processing of the changeset failed. */
    message?: string;
    /** Number of items successfull processed. */
    itemCount: number;
    /** Number of failed request items. */
    errorCount: number;
    /** The collection of nodes changed as a result of the operation. Returned if <= 500 items echoed. */
    nodes?: NodeResponse[];
    /** Url for downloading echoed processed items. Returned if > 500 items echoed (for example deleted nodes in snapshot mode).
     * File contains multiple separate NodeResponse objects in Newline Delimited JSON format (see http://ndjson.org/, Content-Type=application/x-ndjson).
     */
    resultsUrl?: string;
    /** The collection of nodes that were left unprocessed because of errors. */
    unprocessed?: UnprocessedNode[];
    /** Url for downloading failed request items. Returned if > 500 items failed (for example failed deletion of nodes in snapshot mode).
     * File contains multiple separate UnprocessedNode objects in Newline Delimited JSON format (see http://ndjson.org/, Content-Type=application/x-ndjson).
     */
    errorsUrl?: string;
}
/**
 * The node unprocessed due to some error.
 */
export interface UnprocessedNode {
    /** The node change requested in the changeset explicitly or implicitly (e.g. node not mentioned in the snaphot type of changeset request). */
    item: ChangeNodeRequest;
    /** The error code. */
    code: string;
    /** The error message. */
    message: string;
}
/**
 * The response body for initializing asynchronous changeset,  @see Organizer#changesetAsync operation.
 */
export interface OrgChangesetResponseAsync extends OrgChangesetResponseAccepted {
    /** Url for uploading request items. File must contain multiple separate ChangeNodeRequest objects in Newline Delimited JSON format (see http://ndjson.org/, Content-Type=application/x-ndjson). */
    uploadUrl: string;
}
/**
 * The response body for @see Organizer#changesetStatus operation.
 */
export interface OrgChangesetStatusResponse extends OrgChangesetResponseAccepted {
    /** The timestamp in ISO format when processing has been started. */
    startedAt?: string;
    /** The timestamp in ISO format when processing has been finished. */
    processedAt?: string;
    /** The error message if processing of the changeset failed. */
    message?: string;
    /** Number of items successfull processed. */
    nodeCount: number;
    /** Number of failed request items. */
    unprocessedCount: number;
    /** Url for downloading echoed processed items. File contains multiple separate NodeResponse objects in Newline Delimited JSON format (see http://ndjson.org/, Content-Type=application/x-ndjson). */
    resultsUrl?: string;
    /** Url for downloading failed request items. File contains multiple separate UnprocessedNode objects in Newline Delimited JSON format (see http://ndjson.org/, Content-Type=application/x-ndjson). */
    errorsUrl?: string;
}
/**
 * The service client that represents connection to the Organizer Service.
 * Each API operation is exposed as a function on service.
 */
export declare class Organizer extends Service {
    /**
     * @constructor Constructs a service object.
     * @param {Config} config The configuration options (e.g. service url). If not provided the default configuration will be used.
     * @param {Credentials} credentials The credentials to be attached to service requests.
     */
    constructor(config?: Partial<Configuration>);
    /**
     * Returns information the service knows about the current user.
     * @returns {Promise<ServiceResponse<UserInfoResponse>>} the information known about the caller.
     */
    getUserInfo(): Promise<ServiceResponse<UserInfoResponse>>;
    /**
     * Returns some or all (up to 500, default is 100) of the trees in a forest.
     * You can use the request parameters as selection criteria to return a subset of the trees in a forest.
     * @param {string} forestId The identifier of the forest to list trees from.
     * @param {ListTreesOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListTreesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listTrees(forestId: string, params?: ListTreesOptions): Promise<ServiceResponse<ListTreesResponse>>;
    /**
     * Returns the tree descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {GetTreeOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    getTree(forestId: string, treeId: string, params?: GetTreeOptions): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Creates a new tree.
     * @param {string} forestId The identifier of the forest to create a tree in.
     * @param {CreateTreeRequest} request The tree creation request.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    createTree(forestId: string, request: CreateTreeRequest): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Updates the tree descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {UpdateTreeRequest} request The tree descriptor update request.
     * @returns {Promise<ServiceResponse<TreeResponse>>} The service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    updateTree(forestId: string, treeId: string, request: UpdateTreeRequest): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Marks the tree as deleted.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @returns {Promise<TreeResponse>>} The de-serialized service response with tree descriptor.
     * @param {DeleteOptions} params The optional request parameters.
     * @throws {ServiceError} in case of the error response from the service.
     */
    deleteTree(forestId: string, treeId: string, params?: DeleteOptions): Promise<ServiceResponse<TreeResponse>>;
    /**
     * Get the tree access control policy.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @returns {Promise<ServiceResponse<Policy>>} the access control policy.
     */
    getTreePolicy(forestId: string, treeId: string): Promise<ServiceResponse<Policy>>;
    /**
     * Puts the tree access control policy.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {Policy} policy The policy.
     * @returns {Promise<ServiceResponse<void>>} the empty response.
     */
    putTreePolicy(forestId: string, treeId: string, policy: Policy): Promise<ServiceResponse<void>>;
    /**
     * Returns some or all (up to 500, default is 100) of the nodes in a tree.
     * You can use the request parameters as selection criteria to return a subset of the nodes in a tree.
     * @param {string} forestId The identifier of the forest.
     * @param {string} treeId The identifier of the tree to list trees from.
     * @param {ListNodesOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListNodesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listNodes(forestId: string, treeId: string, params?: ListNodesOptions): Promise<ServiceResponse<ListNodesResponse>>;
    /**
     * Returns some or all (up to 500, default is 100) of the node versions.
     * @param {string} forestId The identifier of the forest.
     * @param {string} treeId The identifier of the tree to list trees from.
     * @param {string} nodeId The node identifier.
     * @param {ListNodeVersionsOptions} params The request parameters.
     * @returns {Promise<ServiceResponse<ListNodesResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    listNodeVersions(forestId: string, treeId: string, nodeId: string, params?: ListNodeVersionsOptions): Promise<ServiceResponse<ListNodesResponse>>;
    /**
     * Returns the node descriptor.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {GetNodeOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The service response with node descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    getNode(forestId: string, treeId: string, nodeId: string, params?: GetNodeOptions): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Returns the node links.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {GetNodeOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<string[]>>} The service response with array of node links.
     * @throws {ServiceError} in case of the error response from the service.
     */
    getNodeLinks(forestId: string, treeId: string, nodeId: string, params?: GetNodeOptions): Promise<ServiceResponse<string[]>>;
    /**
     * Creates a new node.
     * @param {string} forestId The identifier of the forest to create a tree in.
     * @param {string} treeId The identifier of the forest to create a tree in.
     * @param {CreateNodeRequest} request The tree creation request.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response with tree descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    createNode(forestId: string, treeId: string, node: CreateNodeRequest): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Updates the node.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {UpdateNodeRequest} request The tree descriptor update request.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    updateNode(forestId: string, treeId: string, nodeId: string, node: UpdateNodeRequest): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Marks the node as deleted.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {string} nodeId The node identifier.
     * @param {DeleteOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<NodeResponse>>} The de-serialized service response with node descriptor.
     * @throws {ServiceError} in case of the error response from the service.
     */
    deleteNode(forestId: string, treeId: string, nodeId: string, params?: DeleteOptions): Promise<ServiceResponse<NodeResponse>>;
    /**
     * Batch get nodes inside a single trees.
     * It should never return a paged response.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {BatchGetNodesRequest} request The ids/versions of nodes to return.
     * @param {GetDefinitionOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<BatchGetNodesResponse>>} The service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    batchGetNodes(forestId: string, treeId: string, request: BatchGetNodesRequest, params?: BatchGetOptions): Promise<ServiceResponse<BatchGetResponse>>;
    /**
     * Batch get nodes across multiple trees.
     * It should never return a paged response.
     * @param {BatchGetRequest} request The ids/versions of nodes to return.
     * @param {BatchGetOptions} params The optional request parameters.
     * @returns {Promise<ServiceResponse<BatchGetResponse>>} The service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    batchGet(request: BatchGetRequest, params?: BatchGetOptions): Promise<ServiceResponse<BatchGetResponse>>;
    /**
     * Applies a changeset to a tree.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {OrgChangesetRequest} changeset The request.
     * @returns {Promise<ServiceResponse<OrgChangesetResponseInline>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    changeset(forestId: string, treeId: string, changeset: OrgChangesetRequest, params?: ChangesetOptions): Promise<ServiceResponse<OrgChangesetResponseInline | OrgChangesetResponseAccepted>>;
    /**
     * Initialize a changeset and get a link for uploading changeset for later processing.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The tree identifier.
     * @param {OrgChangesetConfiguration} changesetConfig The configuration for processing the changes.
     * @returns {Promise<ServiceResponse<ChangesetStatusResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    changesetAsync(forestId: string, treeId: string, changesetConfig: OrgChangesetConfiguration): Promise<ServiceResponse<OrgChangesetResponseAsync>>;
    /**
     * Gets a changeset status.
     * @param {string} changesetId The changeset identifier.
     * @returns {Promise<ServiceResponse<ChangesetStatusResponse>>} The de-serialized service response.
     * @throws {ServiceError} in case of the error response from the service.
     */
    changesetStatus(changesetId: string): Promise<ServiceResponse<OrgChangesetStatusResponse>>;
    /**
     * Requests credential to subscribe to change notifications.
     * @returns {Promise<ServiceResponse<WSSubscriptionResponse>>} The de-serialized service response with temporary credentials and subscription details.
     * @throws {ServiceError} in case of the error response from the service.
     */
    subscribeWS(): Promise<ServiceResponse<WSSubscriptionResponse>>;
    /**
     * Requests credential to subscribe to change notifications.
     * @param {string} forestId The forest identifier.
     * @param {string} treeId The optional tree identifier.
     * @param {string} nodeId The optional node identifier.
     * @returns {Promise<ServiceResponse<SubscriptionResponse>>} The de-serialized service response with temporary credentials and subscription details.
     * @throws {ServiceError} in case of the error response from the service.
     */
    subscribe(forestId: string, treeId?: string, nodeId?: string): Promise<ServiceResponse<SubscriptionResponse>>;
}
